package at.jku.ssw.main;

import at.jku.ssw.DotMaker;
import at.jku.ssw.Edge;
import at.jku.ssw.Graph;
import at.jku.ssw.Out;
import at.jku.ssw.Vertex;
import at.jku.students.Graphs;

public class WarshallTest {

    static void makeDot(String filename, Graph graph) {
        Out.open(filename);
        Out.println(DotMaker.makeDotForGraph(graph));
        Out.close();
    }

    public static void main(String[] args) {
        test1();
        test2();
    }

    static void test1() {
    Out.println("\n*** Test 1 - transitive hull (directed, unweighted) ***");
    Graph g = new Graph();
    Vertex a = new Vertex('A', true); g.addVertex(a);
    Vertex b = new Vertex('B');       g.addVertex(b);
    Vertex c = new Vertex('C');       g.addVertex(c);
    Vertex d = new Vertex('D');       g.addVertex(d);
    Vertex e = new Vertex('E');       g.addVertex(e);
    Vertex f = new Vertex('F');       g.addVertex(f);
    g.addEdge(new Edge(a, b, Edge.Kind.Directed));
    g.addEdge(new Edge(a, d, Edge.Kind.Directed));
    g.addEdge(new Edge(a, c, Edge.Kind.Directed));
    g.addEdge(new Edge(c, f, Edge.Kind.Directed));
    g.addEdge(new Edge(d, b, Edge.Kind.Directed));
    g.addEdge(new Edge(d, e, Edge.Kind.Directed));
    g.addEdge(new Edge(d, f, Edge.Kind.Directed));
    g.addEdge(new Edge(e, c, Edge.Kind.Directed));
    g.addEdge(new Edge(f, e, Edge.Kind.Directed));
    makeDot("1a.dot", g);
    int added = Graphs.makeTransitiveHull(g);
    Out.println(String.format("Warshall: added %d nodes", added));
    makeDot("1b.dot", g);
  }

  static void test2() {
    Out.println("\n*** Test 2 - transitive hull (undirected, unweighted) ***");
    Graph g = new Graph();
    Vertex a = new Vertex('A', true); g.addVertex(a);
    Vertex b = new Vertex('B');       g.addVertex(b);
    Vertex c = new Vertex('C');       g.addVertex(c);
    Vertex d = new Vertex('D');       g.addVertex(d);
    g.addEdge(new Edge(a, b, 0,  Edge.Kind.Undirected));
    g.addEdge(new Edge(b, c, 0,  Edge.Kind.Undirected));
    g.addEdge(new Edge(b, d, 0,  Edge.Kind.Undirected));
    g.addEdge(new Edge(c, a, 0,  Edge.Kind.Undirected));
    makeDot("2a.dot", g);
    int added = Graphs.makeTransitiveHull(g);
    Out.println(String.format("Warshall: added %d nodes", added));
    makeDot("2b.dot", g);
  }
}
